#include <errno.h>
#include <ini.h>
#include <libmp/all.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "config.h"
#include "intl.h"
#include "log.h"
#include "util.h"

bool config_set(config_t *config, char *key, char *value) {
  if (eq(key, "tmpdir"))
    config->tmpdir = strdup(value);
  else if (eq(key, "datadir"))
    config->datadir = strdup(value);
  else
    return false;
  return true;
}

pool_config_t *config_pool_add(config_t *config, char *name) {
  pool_config_t *pool = malloc(sizeof(pool_config_t));
  bzero(pool, sizeof(pool_config_t));

  pool->name = strdup(name);
  pool->next = config->pools;

  config->pools = pool;
  config->pool_count++;

  return pool;
}

bool config_pool_contains(config_t *config, char *name) {
  pool_config_t *cur = config->pools;
  while (NULL != cur) {
    if (eq(cur->name, name))
      return true;
    cur = cur->next;
  }

  return false;
}

int config_load_handler(void *data, const char *_section, const char *_key, const char *_value) {
  char     *section = (char *)_section, *key = (char *)_key, *value = (char *)_value;
  config_t *config = data;

  if (eq(section, "")) {
    if (config_set(config, key, value))
      return 1;
    goto unknown;
  }

  if (!config_pool_contains(config, section))
    config_pool_add(config, section);

  if (eq(key, "url")) {
    free(config->pools->url);
    config->pools->url = strdup(value);
  }

  else
    goto unknown;

  return 1;

unknown:
  error(_("Unknown configuration option: %s/%s"), section, key);
  return 0;
}

bool config_load(lm_ctx_t *ctx, config_t *config, char *file) {
  bzero(config, sizeof(config_t));

  if (access(file, R_OK) != 0) {
    error(_("Failed to access the configuration file: " FG_BOLD "%s"), file);
    return false;
  }

  if (ini_parse(file, config_load_handler, config) < 0) {
    error(_("Failed to parse the configuration"));
    return false;
  }

  if (NULL == config->tmpdir) {
    error(_("Please specify \"tmpdir\" in the configuration"));
    return false;
  }

  if (NULL == config->datadir) {
    error(_("Please specify \"datadir\" in the configuration"));
    return false;
  }

  pool_config_t *pool = config->pools;
  while (NULL != pool) {
    if (NULL == pool->url) {
      error(_("URL not specified for the pool: %s"), pool->name);
      return false;
    }
    pool = pool->next;
  }

  return true;
}

void config_free(config_t *config) {
  free(config->datadir);
  free(config->tmpdir);

  pool_config_t *cur = config->pools, *old = NULL;
  while (NULL != cur) {
    old = cur;
    cur = cur->next;

    free(old->name);
    free(old->url);
    free(old);
  }
}
